﻿###############################################################################################################
# 
# Name         : SequencerVersion.ps1
# Author       : Sander Vis - http://www.pepperbyte.com
# Version      : 1.0, June 4 2014
#                - Initial release
#
# Description  : Extracts SequencerVersion information from within a .AppV file
# 
# Dependencies : PowerShell 3.0 and .NET Framework 4.5
#
# Usage        : The script requires a path parameter. The script will search for all .appv files within the
#                provided folder, even if they are in subfolders.
#
# Credits      : Virtual Engine (Iain Brighton) for the VirtualEngine-AppV module and for writing the following
#                blog: http://virtualengine.co.uk/2013/extracting-files-from-an-appv-file-with-powershell/
#
###############################################################################################################

[CmdletBinding()]
Param(

# Create a path parameter and check whether the supplied path is valid.

	[Parameter(Mandatory=$True,HelpMessage="Enter the root path where all .AppV files are saved.")]
    [ValidateScript({
        if(!(Test-Path $_ -PathType Container)) { throw "$_ does not exist." }
        else { return $true }
        })]
    [String]$path
)

# Load necessary .NET assemblies

[System.Reflection.Assembly]::LoadWithPartialName("System.IO.Compression") | Out-Null
[System.Reflection.Assembly]::LoadWithPartialName("System.IO.Compression.FileSystem") | Out-Null

# Retrieve a list of App-V versions

$url = "http://support.microsoft.com/kb/2940578"
$res = Invoke-WebRequest $url
$html = $res.ParsedHtml.body.GetElementsByClassName("table") | Select -ExpandProperty outerHTML

# Create a GUID and a directory to save the extracted XML-files to

$guid = [guid]::NewGuid()
New-Item -ItemType Directory -Force -Path "$env:TEMP\$guid" | Out-Null

# Filter all .appv files from the supplied path

gci $path -Recurse -Filter *.appv | Sort-Object BaseName | % {
    
# Since an .appv file is basically a zipped file, it can be opened as such
    
    $Zip = New-Object System.IO.Compression.ZipArchive(New-Object System.IO.FileStream($_.FullName, [System.IO.FileMode]::Open))

# Get the PackageHistory.xml file and check if it exists

    $ZippedFile = $Zip.GetEntry("PackageHistory.xml")
    if ($ZippedFile -eq $null) {
        Write-Error ("PackageHistory.xml does not exist in the source .AppV file (" + $_.BaseName + ").")
        $Zip.Dispose()
        return $false
    }

# Save the PackageHistory.xml in our temp directory for further processing

    $exportname = $_.BaseName + "_" + $ZippedFile
    $OutFile = Join-Path $env:TEMP "$guid\$exportname"

    [System.IO.Compression.ZipFileExtensions]::ExtractToFile($ZippedFile, $OutFile)

# Close the .appv file

    $Zip.Dispose()

# Get the content of the PackageHistory.xml

    [xml]$xml = Get-Content ("$env:TEMP\$guid\$exportname")

# Start output

    $_.BaseName + ":"

# There might be multiple PackageHistoryItems and we want to see them all, including their information

    $xml.PackageHistory.PackageHistoryItem | % {

# Save the SequencerVersion and the Time (timestamp the file was saved) of the PackageHistoryItem

        $version = $_.SequencerVersion
        $timestamp = [datetime]$_.Time

# Use a regular expression to get the corresponding App-V version

        Write-Output ([regex]::Matches($html, '<td>(App-V.*?)</td><td>(\d{1}.*?)</') | `
            ? { $_.Captures[0].Groups[2].value -eq $version } | `
            % { $_.Captures[0].Groups[1].value + " - " + $timestamp })
    }

# End output with an extra empty line for readability purposes

    "`r"
}

# Clean up the temporary folder by removing it completely

Remove-Item -Path "$env:TEMP\$guid" -Recurse
